@@ -12,6 +12,7 @@ CMplot <- function(
 	cex=c(0.5,1,1),
 	r=0.3,
 	xlab="Chromosome",
+	bin.max=NULL,
 	ylab=expression(-log[10](italic(p))),
 	xaxs="i",
 	yaxs="r",
@@ -31,6 +32,7 @@ CMplot <- function(
 	cir.chr.col="black",
 	cir.legend=TRUE,
 	cir.legend.cex=0.8,
+	bin.size=1e6,
 	cir.legend.col="grey45",
 	LOG10=TRUE,
 	box=FALSE,
@@ -38,9 +40,10 @@ CMplot <- function(
 	conf.int.col="grey",
 	file.output=TRUE,
 	file="jpg",
-	dpi=300
+	dpi=300,
+	memo=""
 )
-{	
+{
 
 	#plot a circle with a radius of r
 	circle.plot <- function(myr,type="l",x=NULL,lty=1,lwd=1,col="black",add=TRUE,n.point=1000)
@@ -57,6 +60,7 @@ CMplot <- function(
 		band=3,
 		width=5,
 		legend.len=10,
+		legend.max=NULL,
 		legend.pt.cex=3,
 		legend.cex=1,
 		legend.y.intersp=1
@@ -88,7 +92,7 @@ CMplot <- function(
 		maxbin.num <- NULL
 		for(i in 1 : length(chr.num)){
 			pos.x[[i]] <- pos[which(chr == chr.num[i])]
-			cut.len <- round((max(pos.x[[i]]) - min(pos.x[[i]])) / bin)
+			cut.len <- ceiling((max(pos.x[[i]]) - min(pos.x[[i]])) / bin)
 			if(cut.len <= 1){
 				col.index[[i]] = 1
 			}else{
@@ -98,12 +102,21 @@ CMplot <- function(
 				col.index[[i]] <- rep(eachbin.num, eachbin.num)
 			}
 		}
-		maxbin.num <- max(maxbin.num)
+		Maxbin.num <- max(maxbin.num)
+		maxbin.num <- Maxbin.num
+		if(!is.null(legend.max)){
+			maxbin.num <- legend.max
+		}
 		col=colorRampPalette(col)(maxbin.num)
 		for(i in 1 : length(chr.num)){
 			polygon(c(min(pos.x[[i]]), min(pos.x[[i]]), max(pos.x[[i]]), max(pos.x[[i]])), 
 				c(-width/5 - band * (i - length(chr.num) - 1), width/5 - band * (i - length(chr.num) - 1), 
 				width/5 - band * (i - length(chr.num) - 1), -width/5 - band * (i - length(chr.num) - 1)), col="grey", border="grey")
+			if(!is.null(legend.max)){
+				if(legend.max < Maxbin.num){
+					col.index[[i]][col.index[[i]] > legend.max] <- legend.max
+				}
+			}
 			segments(pos.x[[i]], -width/5 - band * (i - length(chr.num) - 1), pos.x[[i]], width/5 - band * (i - length(chr.num) - 1), 
 			col=col[round(col.index[[i]] * length(col) / maxbin.num)], lwd=1)
 		}
@@ -122,13 +135,30 @@ CMplot <- function(
 		legend.y <- round(seq(0, maxbin.num, length=legend.len))
 		len <- legend.y[2]
 		legend.y <- seq(0, maxbin.num, len)
-		if(!maxbin.num %in% legend.y){
-			legend.y <- c(legend.y, paste(">", max(legend.y), sep=""))
-			legend.y.col <- c(legend.y[c(-1, -length(legend.y))], maxbin.num)
-			legend.y.col <- as.numeric(legend.y.col)
+		if(!is.null(legend.max)){
+			if(legend.max < Maxbin.num){
+				if(!maxbin.num %in% legend.y){
+					legend.y <- c(legend.y, paste(">=", maxbin.num, sep=""))
+					legend.y.col <- c(legend.y[c(-1, -length(legend.y))], maxbin.num)
+				}else{
+					legend.y[length(legend.y)] <- paste(">=", maxbin.num, sep="")
+					legend.y.col <- c(legend.y[c(-1, -length(legend.y))], maxbin.num)
+				}
+			}else{
+				if(!maxbin.num %in% legend.y){
+					legend.y <- c(legend.y, maxbin.num)
+				}
+				legend.y.col <- c(legend.y[-1])
+			}
 		}else{
-			legend.y.col <- c(legend.y[-1])
+			if(!maxbin.num %in% legend.y){
+				legend.y <- c(legend.y, paste(">", max(legend.y), sep=""))
+				legend.y.col <- c(legend.y[c(-1, -length(legend.y))], maxbin.num)
+			}else{
+				legend.y.col <- c(legend.y[-1])
+			}
 		}
+		legend.y.col <- as.numeric(legend.y.col)
 		legend('bottomright', title="", legend=legend.y, pch=15, pt.cex = legend.pt.cex, col=c("grey", col[round(legend.y.col * length(col) / maxbin.num)]),
 			cex=legend.cex, bty="n", y.intersp=legend.y.intersp, x.intersp=1, yjust=0.5, xjust=0)
 	}
@@ -136,6 +166,9 @@ CMplot <- function(
 	if(sum(plot.type %in% "b")==1) plot.type=c("c","m","q","d")
 	
 	taxa=colnames(Pmap)[-c(1:3)]
+	if(memo != "")	memo <- paste("_", memo, sep="")
+	if(length(taxa) == 0)	taxa <- "Index"
+	taxa <- paste(taxa, memo, sep="")
 	
 	#SNP-Density plot
 	if("d" %in% plot.type){
@@ -148,147 +181,146 @@ CMplot <- function(
 		if(!file.output){
 			dev.new(width=9, height=7)
 		}
-		Densitplot(map=Pmap[,c(1:3)], col=col)
+		Densitplot(map=Pmap[,c(1:3)], col=col, bin=bin.size, legend.max=bin.max, main=paste("The number of SNPs within ", bin.size/1e6, "Mb window size", sep=""))
 		if(file.output)	dev.off()
 	}
-	
-	#order Pmap by the name of SNP
-	#Pmap=Pmap[order(Pmap[,1]),]
-	Pmap <- as.matrix(Pmap)
-	
-	#delete the column of SNPs names
-	Pmap <- Pmap[,-1]
-	Pmap <- na.omit(Pmap)
-	
-	#scale and adjust the parameters
-	cir.chr.h <- cir.chr.h/5
-	cir.band <- cir.band/5
-	if(!is.null(threshold)){
-		threshold.col <- rep(threshold.col,length(threshold))
-		threshold.lwd <- rep(threshold.lwd,length(threshold))
-		threshold.lty <- rep(threshold.lty,length(threshold))
-	}
-	if(length(cex)!=3) cex <- rep(cex,3)
-	if(!is.null(ylim)){
-		if(length(ylim)==1) ylim <- c(0,ylim)
-	}
-	
-	#get the number of traits
-	R=ncol(Pmap)-2
-	
-	if(R == 1)	multracks=FALSE
-	
-	#replace the non-euchromosome
-	options(warn = -1)
-	numeric.chr <- as.numeric(Pmap[, 1])
-	options(warn = 0)
-	max.chr <- max(numeric.chr, na.rm=TRUE)
-	if(is.infinite(max.chr))	max.chr <- 0
-	map.xy.index <- which(!numeric.chr %in% c(0:max.chr))
-	if(length(map.xy.index) != 0){
-		chr.xy <- unique(Pmap[map.xy.index, 1])
-		for(i in 1:length(chr.xy)){
-			Pmap[Pmap[, 1] == chr.xy[i], 1] <- max.chr + i
-		}
-	}
-	
-	Pmap <- matrix(as.numeric(Pmap), nrow(Pmap))
-	
-	#order the GWAS results by chromosome and position
-	Pmap <- Pmap[order(Pmap[, 1], Pmap[,2]), ]
-	
-	#get the index of chromosome
-	chr <- unique(Pmap[,1])
-	chr.ori <- chr
-	if(length(map.xy.index) != 0){
-		for(i in 1:length(chr.xy)){
-			chr.ori[chr.ori == max.chr + i] <- chr.xy[i]
-		}
-	}
-	
-	pvalueT <- as.matrix(Pmap[,-c(1:2)])
-	
-	if(LOG10){
-		pvalueT[pvalueT <= 0] <- 1
-		pvalueT[pvalueT > 1] <- 1
-	}
+	if(length(plot.type) !=1 | (!"d" %in% plot.type)){
+		#order Pmap by the name of SNP
+		#Pmap=Pmap[order(Pmap[,1]),]
+		Pmap <- as.matrix(Pmap)
 
-	#set the colors for the plot
-	#palette(heat.colors(1024)) #(heatmap)
-	#T=floor(1024/max(pvalue))
-	#plot(pvalue,pch=19,cex=0.6,col=(1024-floor(pvalue*T)))
-	if(is.vector(col)){
-		col <- matrix(col,R,length(col),byrow=T)
-	}
-	if(is.matrix(col)){
-		#try to transform the colors into matrix for all traits
-		col <- matrix(as.vector(t(col)),R,dim(col)[2],byrow=T)
-	}
-	
-	Num <- as.numeric(table(Pmap[,1]))
-	Nchr <- length(Num)
-	N <- NULL
-	
-	#set the colors for each traits
-	for(i in 1:R){
-		colx <- col[i,]
-		colx <- colx[!is.na(colx)]
-		N[i] <- ceiling(Nchr/length(colx))
-	}
-	
-	#scale the space parameter between chromosomes
-	if(!missing(band)){
-		band <- floor(band*(dim(pvalueT)[1]/100))
-	}else{
-		band <- floor(dim(pvalueT)[1]/100)
-	}
-	
-	#insert the space into chromosomes and return the midpoint of each chromosome
-	ticks <- NULL
-	NewP <- NULL
-	for(j in 1:R){
-		pvalue <- pvalueT[,j]
-		for(i in 0:(Nchr-1)){
-			if (i==0){
-				pvalue <- append(pvalue,rep(Inf,band),after=0)
-				ticks[i+1] <- band+floor((Num[i+1])/2)
-			}else{
-				pvalue <- append(pvalue,rep(Inf,band),after=sum(Num[1:i])+i*band)
-				ticks[i+1] <- band*(i+1)+sum(Num[1:i])+floor((Num[i+1])/2)
+		#delete the column of SNPs names
+		Pmap <- Pmap[,-1]
+		Pmap <- na.omit(Pmap)
+
+		#scale and adjust the parameters
+		cir.chr.h <- cir.chr.h/5
+		cir.band <- cir.band/5
+		if(!is.null(threshold)){
+			threshold.col <- rep(threshold.col,length(threshold))
+			threshold.lwd <- rep(threshold.lwd,length(threshold))
+			threshold.lty <- rep(threshold.lty,length(threshold))
+		}
+		if(length(cex)!=3) cex <- rep(cex,3)
+		if(!is.null(ylim)){
+			if(length(ylim)==1) ylim <- c(0,ylim)
+		}
+
+		#get the number of traits
+		R=ncol(Pmap)-2
+
+		#replace the non-euchromosome
+		options(warn = -1)
+		numeric.chr <- as.numeric(Pmap[, 1])
+		options(warn = 0)
+		max.chr <- max(numeric.chr, na.rm=TRUE)
+		if(is.infinite(max.chr))	max.chr <- 0
+		map.xy.index <- which(!numeric.chr %in% c(0:max.chr))
+		if(length(map.xy.index) != 0){
+			chr.xy <- unique(Pmap[map.xy.index, 1])
+			for(i in 1:length(chr.xy)){
+				Pmap[Pmap[, 1] == chr.xy[i], 1] <- max.chr + i
 			}
 		}
-		NewP[[j]] <- pvalue
-	}
-	
-	#merge the pvalues of traits by column
-	pvalueT <- do.call(cbind,NewP)
-	if(LOG10 == TRUE){
-		logpvalueT <- -log10(pvalueT)
-	}else{
-		logpvalueT <- pvalueT
-	}
-	
-	Num <- Num+band
-	add <- list()
-	for(i in 1:R){
-		colx <- col[i,]
-		colx <- colx[!is.na(colx)]
-		add[[i]] <- c(Num,rep(0,N[i]*length(colx)-Nchr))
-	}
 
-	TotalN1 <- nrow(pvalueT)
-	TotalN <- TotalN1
+		Pmap <- matrix(as.numeric(Pmap), nrow(Pmap))
 
-	signal.line.index <- NULL
-	if(!is.null(threshold)){
-		if(!is.null(signal.line)){
-			for(l in 1:R){
-				signal.line.index <- c(signal.line.index,which(pvalueT[,l] < min(threshold)/nrow(pvalueT)))
+		#order the GWAS results by chromosome and position
+		Pmap <- Pmap[order(Pmap[, 1], Pmap[,2]), ]
+
+		#get the index of chromosome
+		chr <- unique(Pmap[,1])
+		chr.ori <- chr
+		if(length(map.xy.index) != 0){
+			for(i in 1:length(chr.xy)){
+				chr.ori[chr.ori == max.chr + i] <- chr.xy[i]
+			}
+		}
+
+		pvalueT <- as.matrix(Pmap[,-c(1:2)])
+
+		if(LOG10){
+			pvalueT[pvalueT <= 0] <- 1
+			pvalueT[pvalueT > 1] <- 1
+		}
+
+		#set the colors for the plot
+		#palette(heat.colors(1024)) #(heatmap)
+		#T=floor(1024/max(pvalue))
+		#plot(pvalue,pch=19,cex=0.6,col=(1024-floor(pvalue*T)))
+		if(is.vector(col)){
+			col <- matrix(col,R,length(col),byrow=T)
+		}
+		if(is.matrix(col)){
+			#try to transform the colors into matrix for all traits
+			col <- matrix(as.vector(t(col)),R,dim(col)[2],byrow=T)
+		}
+
+		Num <- as.numeric(table(Pmap[,1]))
+		Nchr <- length(Num)
+		N <- NULL
+
+		#set the colors for each traits
+		for(i in 1:R){
+			colx <- col[i,]
+			colx <- colx[!is.na(colx)]
+			N[i] <- ceiling(Nchr/length(colx))
+		}
+
+		#scale the space parameter between chromosomes
+		if(!missing(band)){
+			band <- floor(band*(dim(pvalueT)[1]/100))
+		}else{
+			band <- floor(dim(pvalueT)[1]/100)
+		}
+
+		#insert the space into chromosomes and return the midpoint of each chromosome
+		ticks <- NULL
+		NewP <- NULL
+		for(j in 1:R){
+			pvalue <- pvalueT[,j]
+			for(i in 0:(Nchr-1)){
+				if (i==0){
+					pvalue <- append(pvalue,rep(Inf,band),after=0)
+					ticks[i+1] <- band+floor((Num[i+1])/2)
+				}else{
+					pvalue <- append(pvalue,rep(Inf,band),after=sum(Num[1:i])+i*band)
+					ticks[i+1] <- band*(i+1)+sum(Num[1:i])+floor((Num[i+1])/2)
+				}
+			}
+			NewP[[j]] <- pvalue
+		}
+
+		#merge the pvalues of traits by column
+		pvalueT <- do.call(cbind,NewP)
+		if(LOG10 == TRUE){
+			logpvalueT <- -log10(pvalueT)
+		}else{
+			pvalueT <- abs(pvalueT)
+			logpvalueT <- pvalueT
+		}
+
+		Num <- Num+band
+		add <- list()
+		for(i in 1:R){
+			colx <- col[i,]
+			colx <- colx[!is.na(colx)]
+			add[[i]] <- c(Num,rep(0,N[i]*length(colx)-Nchr))
+		}
+
+		TotalN1 <- nrow(pvalueT)
+		TotalN <- TotalN1
+
+		signal.line.index <- NULL
+		if(!is.null(threshold)){
+			if(!is.null(signal.line)){
+				for(l in 1:R){
+					signal.line.index <- c(signal.line.index,which(pvalueT[,l] < min(threshold)/nrow(pvalueT)))
+				}
+				signal.line.index <- unique(signal.line.index)
 			}
-			signal.line.index <- unique(signal.line.index)
 		}
 	}
-
 	#plot circle Manhattan
 	if("c" %in% plot.type){
 		#print("Starting Circular-Manhattan plot!",quote=F)
@@ -656,14 +688,14 @@ CMplot <- function(
 					}
 					if(is.null(ylim)){
 						if(Max>1){
-							#print(seq(0,(Max+1),round((Max+1)/10)))
-							axis(2,at=seq(0,(Max+1),round((Max+1)/10)),cex.axis=cex.axis,font=2,labels=seq(0,(Max+1),round((Max+1)/10)))
+							#print(seq(0,(Max+1),ceiling((Max+1)/10)))
+							axis(2,at=seq(0,(Max+1),ceiling((Max+1)/10)),cex.axis=cex.axis,font=2,labels=seq(0,(Max+1),ceiling((Max+1)/10)))
 						}else{
 							axis(2,at=seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))),cex.axis=cex.axis,font=2,labels=seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))))
 						}
 					}else{
 						if(ylim[2]>1){
-							axis(2,at=seq(0,(ylim[2]+1),round((ylim[2]+1)/10)),cex.axis=cex.axis,font=2,labels=seq(0,(ylim[2]+1),round((ylim[2]+1)/10)))
+							axis(2,at=seq(0,(ylim[2]+1),ceiling((ylim[2]+1)/10)),cex.axis=cex.axis,font=2,labels=seq(0,(ylim[2]+1),ceiling((ylim[2]+1)/10)))
 						}else{
 							axis(2,at=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))),cex.axis=cex.axis,font=2,labels=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))))
 						}
@@ -705,7 +737,7 @@ CMplot <- function(
 				if(file=="jpg")	jpeg(paste("Multracks.Rectangular-Manhattan.",paste(taxa,collapse="."),".jpg",sep=""), width = 14*dpi,height=5*dpi*R,res=dpi,quality = 100)
 				if(file=="pdf")	pdf(paste("Multracks.Rectangular-Manhattan.",paste(taxa,collapse="."),".pdf",sep=""), width = 15,height=6*R)
 				if(file=="tiff")	tiff(paste("Multracks.Rectangular-Manhattan.",paste(taxa,collapse="."),".tiff",sep=""), width = 14*dpi,height=5*dpi*R,res=dpi)
-				par(mfcol=c(R,1),mar=c(1, 5, 1, 2),oma=c(4,0,3,0),xaxs=xaxs,yaxs=yaxs)
+				par(mfcol=c(R,1),mar=c(0, 6+(R-1)*2, 0, 2),oma=c(4,0,4,0),xaxs=xaxs,yaxs=yaxs)
 			}
 			if(file.output==FALSE){
 				dev.new(width = 15, height = 6)
@@ -748,37 +780,44 @@ CMplot <- function(
 							# }
 						}
 					}
+					xn <- ifelse(R == 1, R, R * 2/3)
 					if(Max<=1){
-						plot(logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,length(logpvalue)+band),ylim=c(0,Max+10^(-ceiling(-log10(Max)))),ylab=ylab,
-							cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE)
+						plot(logpvalue,pch=pch,cex=cex[2]*xn,col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,length(logpvalue)+band),ylim=c(0,Max+10^(-ceiling(-log10(Max)))),ylab=ylab,
+							cex.axis=cex.axis*xn,cex.lab=2*xn,font=2,axes=FALSE)
 					}else{
-						plot(logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,length(logpvalue)+band),ylim=c(0,Max+1),ylab=ylab,
-							cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE)
+						plot(logpvalue,pch=pch,cex=cex[2]*xn,col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,length(logpvalue)+band),ylim=c(0,Max+1),ylab=ylab,
+							cex.axis=cex.axis*xn,cex.lab=2*xn,font=2,axes=FALSE)
 					}
 				}else{
-					plot(logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,length(logpvalue)+band),ylim=ylim,ylab=ylab,
-						cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE)
+					plot(logpvalue,pch=pch,cex=cex[2]*xn,col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,length(logpvalue)+band),ylim=ylim,ylab=ylab,
+						cex.axis=cex.axis*xn,cex.lab=2*xn,font=2,axes=FALSE)
 				}
 				
-				#add the names of traits on plot  
-				text(ticks[1],Max,labels=taxa[i],adj=0,font=3,cex=1.5)
-				if(is.null(chr.labels)){
-					axis(1, at=c(0,ticks),cex.axis=cex.axis,font=2,labels=c("Chr",chr.ori))
+				#add the names of traits on plot 
+				if(Max<=1){
+					text(ticks[1],Max+10^(-ceiling(-log10(Max))),labels=taxa[i],adj=0,font=3,cex=xn)
 				}else{
-					axis(1, at=c(0,ticks),cex.axis=cex.axis,font=2,labels=c("Chr",chr.labels))
+					text(ticks[1],Max+1,labels=taxa[i],adj=0,font=3,cex=xn)
 				}
-				if(i==1) mtext("Manhattan plot",side=3,padj=-1,font=2,cex=1.5)
+				if(i == R){
+					if(is.null(chr.labels)){
+						axis(1, at=c(0,ticks),cex.axis=cex.axis*xn,font=2,labels=c("Chr",chr.ori),padj=(xn-1)/2)
+					}else{
+						axis(1, at=c(0,ticks),cex.axis=cex.axis*xn,font=2,labels=c("Chr",chr.labels),padj=(xn-1)/2)
+					}
+				}
+				if(i==1) mtext("Manhattan plot",side=3,padj=-1,font=2,cex=xn)
 				if(is.null(ylim)){
 					if(Max>1){
-						axis(2,at=seq(0,(Max+1),round((Max+1)/10)),cex.axis=cex.axis,font=2,labels=seq(0,(Max+1),round((Max+1)/10)))
+						axis(2,at=seq(0,(Max+1),ceiling((Max+1)/10)),cex.axis=cex.axis*xn,font=2,labels=seq(0,(Max+1),ceiling((Max+1)/10)))
 					}else{
-						axis(2,at=seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))),cex.axis=cex.axis,font=2,labels=seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))))
+						axis(2,at=seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))),cex.axis=cex.axis*xn,font=2,labels=seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))))
 					}
 				}else{
 					if(ylim[2]>1){
-						axis(2,at=seq(0,(ylim[2]+1),round((ylim[2]+1)/10)),cex.axis=cex.axis,font=2,labels=seq(0,(ylim[2]+1),round((ylim[2]+1)/10)))
+						axis(2,at=seq(0,(ylim[2]+1),ceiling((ylim[2]+1)/10)),cex.axis=cex.axis*xn,font=2,labels=seq(0,(ylim[2]+1),ceiling((ylim[2]+1)/10)))
 					}else{
-						axis(2,at=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))),cex.axis=cex.axis,font=2,labels=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))))
+						axis(2,at=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))),cex.axis=cex.axis*xn,font=2,labels=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))))
 					}
 				}
 				if(!is.null(threshold)){
@@ -793,11 +832,11 @@ CMplot <- function(
 							HX1=which(logpvalue>=sgline1)
 							
 							#cover the points that exceed the threshold with the color "white"
-							points(HX1,HY1,pch=pch,cex=cex[2],col="white")
+							points(HX1,HY1,pch=pch,cex=cex[2]*xn,col="white")
 							if(is.null(signal.col)){
-								points(HX1,HY1,pch=signal.pch,cex=signal.cex*cex[2],col=rep(rep(colx,N[i]),add[[i]])[HX1])
+								points(HX1,HY1,pch=signal.pch,cex=signal.cex*cex[2]*xn,col=rep(rep(colx,N[i]),add[[i]])[HX1])
 							}else{
-								points(HX1,HY1,pch=signal.pch,cex=signal.cex*cex[2],col=signal.col)
+								points(HX1,HY1,pch=signal.pch,cex=signal.cex*cex[2]*xn,col=signal.col)
 							}
 						}
 					}
@@ -806,7 +845,7 @@ CMplot <- function(
 			}
 			
 			#add the labels of X-axis
-			mtext(xlab,side=1,padj=2.5,font=2,cex=1.5)
+			#mtext(xlab,side=1,padj=2.5,font=2,cex=R*2/3)
 			if(file.output==TRUE) dev.off()
 			#print("Rectangular-Manhattan has been finished!",quote=F)
 		}
@@ -890,22 +929,24 @@ CMplot <- function(
 			if(file.output==TRUE) dev.off()
 			if(R > 1){
 				if(file.output==TRUE){
-					if(file=="jpg")	jpeg(paste("Multraits.QQplot.",taxa[i],".jpg",sep=""), width = 5.5*dpi,height=5.5*dpi,res=dpi,quality = 100)
-					if(file=="pdf")	pdf(paste("Multraits.QQplot.",taxa[i],".pdf",sep=""), width = 5.5,height=5.5)
-					if(file=="tiff")	tiff(paste("Multraits.QQplot.",taxa[i],".tiff",sep=""), width = 5.5*dpi,height=5.5*dpi,res=dpi)
+					if(file=="jpg")	jpeg(paste("Multraits.QQplot.",paste(taxa,collapse="."),".jpg",sep=""), width = 5.5*dpi,height=5.5*dpi,res=dpi,quality = 100)
+					if(file=="pdf")	pdf(paste("Multraits.QQplot.",paste(taxa,collapse="."),".pdf",sep=""), width = 5.5,height=5.5)
+					if(file=="tiff")	tiff(paste("Multraits.QQplot.",paste(taxa,collapse="."),".tiff",sep=""), width = 5.5*dpi,height=5.5*dpi,res=dpi)
 					par(mar = c(5,5,4,2))
 				}else{
 					dev.new(width = 5.5, height = 5.5)
 				}
 				p_value_quantiles=(1:nrow(Pmap))/(nrow(Pmap)+1)
 				log.Quantiles <- -log10(p_value_quantiles)
-				plot(NULL, xlim = c(0,max(log.Quantiles)), cex.axis=cex.axis, cex.lab=1.2,ylim=c(0,-log10(min(Pmap[,3:(R+2)]))),xlab =expression(Expected~~-log[10](italic(p))), ylab = expression(Observed~~-log[10](italic(p))), main = "QQplot")
+				Pmap.min <- Pmap[,3:(R+2)]
+				YlimMax <- -log10(min(Pmap.min[Pmap.min > 0]))
+				plot(NULL, xlim = c(0,max(log.Quantiles)), cex.axis=cex.axis, cex.lab=1.2,ylim=c(0,YlimMax),xlab =expression(Expected~~-log[10](italic(p))), ylab = expression(Observed~~-log[10](italic(p))), main = "QQplot")
 				for(i in 1:R){
 					print(paste("Multraits_QQ Plotting ",taxa[i],"...",sep=""))
 					P.values=as.numeric(Pmap[,i+2])
 					P.values=P.values[!is.na(P.values)]
 					if(LOG10==TRUE){
-						P.values=P.values[P.values>0]
+						P.values=P.values[P.values>=0]
 						P.values=P.values[P.values<=1]
 						N=length(P.values)
 						P.values=P.values[order(P.values)]
